; Name:    NexOS Bootloader - Switches into PMode and loads the kernel
; Version: 0.1.3 (08/11/2000)
; Author:  Torben Schulz (torben@fasthack.de)
; Website: http://caffeine.tsx.org
;
; This program is distributed in the hope that it will be useful, but WITHOUT ANY
; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
; PARTICULAR PURPOSE. The author is not responsible for any damages caused by
; this program. You use it completely on your own risk.
;
; This program is free software; you can redistribute it and/or modify it under
; the terms of the GNU General Public License as published by the Free Software
; Foundation; either version 2 of the License, or (at your option) any later
; version. See the GNU General Public License for more details. You should have
; received a copy of the GNU General Public License along with this program; if
; not, look at the website http://www.gnu.org/copyleft/gpl.html or write to the
; Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
; MA 02111-1307, USA.

[BITS 16]

jmp starthere

; global descriptor table

dummy_seg_length0_15  dw 0x0000                  ; low word of the segment length
dummy_base_addr0_15   dw 0x0000                  ; low word of base address
dummy_base_addr16_23  db 0x00                    ; low byte of high word of base addr.
dummy_flags           db 0x00                    ; segment type and misc. flags
dummy_access          db 0x00                    ; highest nibble of segment length and access flags (0xCF 32bit)
dummy_base_addr24_31  db 0x00                    ; high byte of high word of base addr. 

code_seg_length0_15   dw 0xFFFF
code_base_addr0_15    dw 0x0000
code_base_addr16_23   db 0x01
code_flags            db 0x9A
code_access           db 0xCF
code_base_addr24_31   db 0x00

data_seg_length0_15   dw 0xFFFF
data_base_addr0_15    dw 0x0000
data_base_addr16_23   db 0x01
data_flags            db 0x92
data_access           db 0xCF
data_base_addr24_31   db 0x00

screen_seg_length0_15 dw 0xFFFF
screen_base_addr0_15  dw 0x8000
screen_base_addr16_23 db 0x0B
screen_flags          db 0x92
screen_access         db 0x00
screen_base_addr24_31 db 0x00

gdt_desc dw 0x0020                               ; gdt size in byte
         dd dummy_seg_length0_15+0x7C00          ; gdt physical address

; ### data and functions ###

bootmsg  db "Caffeine NexOS Bootloader 0.1.3 ",224," (tron)",10,10,13
         db "Loading kernel into memory...................."
checkmsg db "Checking for an 80386 (or better) processor..."
pmodemsg db "Switching into protected mode................."
donemsg  db "done.",10,13
failmsg  db "failed."

printmsg:                                        ; print a message
  push   cx
  mov    ax,0x0300
  mov    bx,0x0007
  int    0x10
  pop    cx
  mov    ax,0x1301
  int    0x10
ret

error:
  mov    ax,cs
  mov    es,ax
  mov    cx,0007                                 ; print message
  mov    bp,0x7C00+failmsg
  call   printmsg  
  endless:
  jmp endless
ret

; ### initialize environment ###

starthere:
  mov    ax,cs                                   ; initalize registers
  cli
  mov    ss,ax
  mov    ds,ax
  mov    es,ax
  mov    sp,0xFFFE
  sti
  mov    ax,0x0003                               ; initialize screen
  int    0x10
  mov    ax,0x1112
  mov    bl,0x00
  int    0x10
  mov    cx,0089                                 ; print message
  mov    bp,0x7C00+bootmsg
  call   printmsg

; ### load kernel module ###

  push    es
  initstart:                                     ; initialize controller
    mov    ah,0x00
    mov    dl,0x00
    int    0x13
  jc initstart
  readstart:                                     ; load sector to 0x1000:0x0000
    mov    ax,0x0201
    mov    cx,0x0002
    mov    dx,0x0000
    mov    bx,0x1000
    mov    es,bx
    mov    bx,0x0000
    int    0x13
  jc readstart
  pop    es
  mov    cx,0007                                 ; print message
  mov    bp,0x7C00+donemsg
  call   printmsg
  
; ### check for 386+ cpu ###

  mov    cx,0046                                 ; print message
  mov    bp,0x7C00+checkmsg
  call   printmsg
  pushf
  xor    ah,ah                                   ; check for 8086 (bit 12-15 set)
  push   ax
  popf
  pushf
  pop    ax
  and    ah,0xF0 
  cmp    ah,0xF0
  je     error
  mov    ah,0xF0                                 ; check for 286 (bit 12-15 clear)
  push   ax
  popf
  pushf
  pop    ax
  and    ah,0xF0
  jz near error
  popf
  mov    cx,0007                                 ; print message
  mov    bp,0x7C00+donemsg
  call   printmsg
  
; ### switch into pmode ###

  mov    cx,0046                                 ; print message
  mov    bp,0x7C00+pmodemsg
  call   printmsg
  cli
  lgdt   [ds:gdt_desc+7C00h]                     
  mov    eax,cr0
  or     al,1
  mov    cr0,eax
  db 0eah                                        ; far jump to kernel
  dw 0,08h

times (510-($-$$)) db 0                     ; write boot signature
db 0x55,0xAA